#!/bin/sh
# ---------------------------------------------------------------------------
# JOnAS: Java(TM) Open Application Server
# Copyright (C) 1999-2008 Bull S.A.S.
# Contact: jonas-team@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Philippe Durieux
# Contributor(s): ______________________________________.
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------


# -----------------------------------------------------------------------
# Check JONAS_ROOT is set.
# -----------------------------------------------------------------------
if [ ! -d $JONAS_ROOT/ejbjars ]
then
    echo "JONAS_ROOT must be set"
    exit 1
fi

# Set CLASSPATH and JONAS_BASE
. $JONAS_ROOT/bin/unix/setenv

JONAS_LIB=$JONAS_ROOT/lib

if [ ! -f $JONAS_LIB/client.jar ]
then
    echo "client.jar file must be present in JONAS_ROOT/lib directory"
    exit 1
fi


# ---------------------------------------------
# Get args
# ---------------------------------------------
ARGS=
while [ "$#" -gt 0 ]
do      case "$1" in
        -cp)
                shift
                CLASSPATH=$CLASSPATH$SPS$1
                ;;
        -D*)
                JAVA_OPTS="$JAVA_OPTS $1"
                ;;
        -security)
        		# Use security manager for jclient ?
        		JAVA_OPTS="$JAVA_OPTS -Djava.security.manager -Djava.security.policy=$JONAS_BASE/conf/java.policy"
        		;;
        -debug)
                shift
                # $1=-p
                if [ "$1" != "-p" ]; then
                  echo "-debug mode need a following \"-p <debug-port>\""
                  exit 1
                fi
                shift
                JONAS_DEBUG_PORT=$1
                JONAS_DEBUG_SUSPEND=n
                if [ "$2" = "-s" ]; then
                  shift
                  shift
                  JONAS_DEBUG_SUSPEND=$1
                fi
                echo "JOnAS Debug Info :"
                echo " listening on port : $JONAS_DEBUG_PORT"
                echo " suspend mode : $JONAS_DEBUG_SUSPEND"
                JONAS_DEBUG_OPTS=" -Xdebug -Xnoagent -Xrunjdwp:transport=dt_socket,server=y,address=$JONAS_DEBUG_PORT,suspend=$JONAS_DEBUG_SUSPEND"
                ;;
        -argclient)
                break
                ;;
        *)
                # all other args are passed "as is" to the java program
                ARGS="$ARGS $1"
                ;;
        esac
        shift
done

while [ "$#" -gt 0 ]
do
  ARGS="$ARGS $1"
  shift
done

JAVA_OPTS="-Djava.security.auth.login.config=$JONAS_BASE/conf/jaas.config $JAVA_OPTS"
JAVA_OPTS="-Djava.endorsed.dirs=$JONAS_ROOT/lib/endorsed $JAVA_OPTS"


# Add client.jar
# CLASSPATH could be upgraded here
CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/client.jar


$JAVA $JAVA_OPTS $JONAS_DEBUG_OPTS org.objectweb.jonas.client.ClientContainer $ARGS

