/**
 * High Availability Service (HA) for JOnAS
 *
 * Copyright (C) 2006 Distributed Systems Lab.
 * Universidad Polit??cnica de Madrid (Spain)
 * Contact: http://lsd.ls.fi.upm.es/lsd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.ha.interceptor;

import java.util.Stack;

import org.objectweb.carol.cmi.ha.RequestId;
import org.objectweb.carol.cmi.ha.ViewIdWrapper;

/**
 * This is the class stored in the Thread local variable of HACurrent class and
 * represents the thread associated HA context
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 */
public class HACurrentContext {
    /**
     * Holds info about the request chain
     */
    private Stack requests = new Stack();

    /**
     * Holds the on failover info
     */
    private boolean onFailover = false;

    /**
     * client ViewIdWrapper
     * We need to separate client and server view when both
     * sides are collocated
     */
    private ViewIdWrapper clientViewIdWrapper = null;

    /**
     * server ViewIdWrapper
     */
    private ViewIdWrapper serverViewIdWrapper = null;

    /**
     * Constructor
     */
    public HACurrentContext() {
    }

    /**
     * Set the requests
     * @param reqs the requests
     */
    public void setRequests(Stack reqs) {
        requests = reqs;
    }

    /**
     * Get the requests
     * @return the requests
     */
    public Stack getRequests() {
        return requests;
    }

    /**
     * Put the request as the las request
     */
    public void putNextReq(RequestId nextReq) {
        // Check that the request is not in the Stack
        if (requests.contains(nextReq)) {
            while (!requests.peek().equals(nextReq)) {
                requests.pop();
            }
        } else {
            requests.push(nextReq);
        }
    }

    /**
     * @return Returns the onFailover.
     */
    public boolean isOnFailover() {
        return onFailover;
    }

    /**
     * @param onFailover The onFailover to set.
     */
    public void setOnFailover(boolean onFailover) {
        this.onFailover = onFailover;
    }

    /**
     * Get the client view id wrapper
     * @return view id wrapper
     */
    public ViewIdWrapper getClientViewIdWrapper() {
        return clientViewIdWrapper;
    }

    /**
     * Set the client view id wrapper
     * @param clientViewIdWrapper view id wrapper
     */
    public void setClientViewIdWrapper(ViewIdWrapper clientViewIdWrapper) {
        this.clientViewIdWrapper = clientViewIdWrapper;
    }

    /**
     * Get the server view id wrapper
     * @return view id wrapper
     */
    public ViewIdWrapper getServerViewIdWrapper() {
        return serverViewIdWrapper;
    }

    /**
     * Set the server view id wrapper
     * @param serverViewIdWrapper view id wrapper
     */
    public void setServerViewIdWrapper(ViewIdWrapper serverViewIdWrapper) {
        this.serverViewIdWrapper = serverViewIdWrapper;
    }
}
