/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 France Telecom R&D
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jndi.interceptors.impl.datasource;

import java.lang.reflect.Method;

import javax.sql.DataSource;

import org.objectweb.carol.jndi.intercept.ContextInterceptor;
import org.objectweb.carol.jndi.intercept.InterceptionContext;
import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.jndi.interceptors.impl.AbsContextInterceptor;
import org.objectweb.util.monolog.api.Logger;

/**
 * Context interceptor for Datasources
 * @author Florent Benoit
 */
public class DataSourceLeakDetectorContextInterceptor extends AbsContextInterceptor implements ContextInterceptor {

    /**
     * Logger.
     */
    private static Logger logger = Log.getLogger(DataSourceLeakDetectorContextInterceptor.class.getName());


    /**
     * Intercept the lookup on the Datasources.
     * @param context the interception context with data
     * @return the wrapped datasource if intercepted, else it provides the original datasource
     * @throws Exception if interception fails
     */
    public Object intercept(final InterceptionContext context) throws Exception {

        // Invoke the call in order to get the result
        Object obj = context.proceed();

        // Do some stuff if we're interested in a give method
        Method method = context.getMethod();

        // Only react on lookup/lookupLink methods
        if (filterOnMethodName(method)) {

            // And if this is a managed JNDI name
            if (filterOnJndiName(context.getParameters()[0].toString())) {
                if (DataSource.class.isAssignableFrom(obj.getClass())) {

                    // Cast the DS
                    DataSource ds = (DataSource) obj;

                    // Wrap datasource
                    DatasourceWrapper detector = new DatasourceWrapper(ds);

                    // Register it
                    getResourceCheckerManager().enlistResource(detector);

                    // Return our detector DataSource
                    return detector;
                }
            }
        }

        // Let the Object unchanged
        return obj;
    }


}
